@@ -270,6 +270,30 @@ def _get_caller_frame():
     return f
 
 
+def _prompt_continue_waiting_for_debugger():
+    """
+    Prompt while exiting the debugger to get user opinion on keeping the
+    process waiting for debugger to attach.
+    """
+    count_invalid = 0
+    max_invalid_entries = 3
+    while count_invalid < max_invalid_entries:
+        sys.stdout.flush()
+        response = input("Keep the process running for debugger to be "
+                            "attached later ? (y)es/(n)o\n")
+
+        if response.lower() in ('n', 'no'):
+            global _waiting_for_debugger
+            _waiting_for_debugger = None
+            break
+
+        if response.lower() not in ('y', 'yes'):
+            print("Invalid response: {}".format(repr(response)))
+            count_invalid += 1
+        else:
+            break
+
+
 def _debug_exception(*exc_info, **kwargs):
     """
     Debug an exception -- print a stack trace and enter the debugger.
@@ -278,6 +302,7 @@ def _debug_exception(*exc_info, **kwargs):
     """
     from pyflyby._interactive import print_verbose_tb
     tty = kwargs.pop("tty", "/dev/tty")
+    debugger_attached = kwargs.pop("debugger_attached", False)
     if kwargs:
         raise TypeError("debug_exception(): unexpected kwargs %s"
                         % (', '.join(sorted(kwargs.keys()))))
@@ -293,7 +318,13 @@ def _debug_exception(*exc_info, **kwargs):
         # will cause print_verbose_tb to include a line with just a colon.
         # TODO: avoid that line.
         exc_info = ("", "", exc_info)
+
     with _DebuggerCtx(tty=tty) as pdb:
+        if debugger_attached:
+            # If debugger is attached to the process made waiting by
+            # 'wait_for_debugger_to_attach', check with the user whether to
+            # keep the process waiting for debugger to attach.
+            pdb.postloop = _prompt_continue_waiting_for_debugger
         print_verbose_tb(*exc_info)
         pdb.interaction(None, exc_info[2])
 
@@ -437,6 +468,7 @@ def debugger(*args, **kwargs):
     locals          = kwargs.pop("locals"         , None)
     wait_for_attach = kwargs.pop("wait_for_attach", Ellipsis)
     background      = kwargs.pop("background"     , False)
+    _debugger_attached = False
     if kwargs:
         raise TypeError("debugger(): unexpected kwargs %s"
                         % (', '.join(sorted(kwargs))))
@@ -445,7 +477,7 @@ def debugger(*args, **kwargs):
         # (whether it's a frame, traceback, etc).
         global _waiting_for_debugger
         arg = _waiting_for_debugger
-        _waiting_for_debugger = None
+        _debugger_attached = True
     if arg is None:
         # Debug current frame.
         arg = _CURRENT_FRAME
@@ -476,7 +508,15 @@ def debugger(*args, **kwargs):
         return
     if (isinstance(arg, TracebackType) or
         type(arg) is tuple and len(arg) == 3 and type(arg[2]) is TracebackType):
-        _debug_exception(arg, tty=tty)
+        _debug_exception(arg, tty=tty, debugger_attached=_debugger_attached)
+        on_continue()
+        return
+    import threading
+    # If `arg` is an instance of `tuple` that contains
+    # `threading.ExceptHookArgs`, extract the exc_info from it.
+    if type(arg) is tuple and len(arg) == 1 and type(arg[0]) is threading.ExceptHookArgs:
+        arg = arg[0][:3]
+        _debug_exception(arg, tty=tty, debugger_attached=_debugger_attached)
         on_continue()
         return
     if not isinstance(arg, FrameType):
@@ -851,7 +891,7 @@ def enable_sigterm_handler(on_existing_handler='raise'):
         return
     elif on_existing_handler == "raise":
         raise ValueError(
-            "enable_sigterm_handler(on_existing_handler='raise'): SIGTERM handler already exists")
+            "enable_sigterm_handler(on_existing_handler='raise'): SIGTERM handler already exists" + repr(old_handler))
     else:
         raise ValueError(
             "enable_sigterm_handler(): SIGTERM handler already exists, "
@@ -935,7 +975,7 @@ def _escape_for_gdb(string):
         if char in _gdb_safe_chars:
             result.append(char)
         else:
-            result.append("\\%s" % (oct(ord(char)),))
+            result.append(r"\0{0:o}".format(ord(char)))
     return ''.join(result)
 
 
@@ -1018,7 +1058,6 @@ def inject(pid, statements, wait=True, show_gdb_output=False):
     else:
         return process.pid
 
-
 import tty
 
 
@@ -1075,8 +1114,16 @@ def process_exists(pid):
     :rtype:
       ``bool``
     """
+    import psutil
+
     try:
         os.kill(pid, 0)
+
+        # Zombie processes should be treated as non-existing.
+        proc = psutil.Process(pid)
+        if proc.status() == psutil.STATUS_ZOMBIE:
+            return False
+
         return True
     except OSError as e:
         if e.errno == errno.ESRCH:
@@ -1164,6 +1211,7 @@ def attach_debugger(pid):
         while True:
             try:
                 if not process_exists(gdb_pid):
+                    print("GDB process has exited.")
                     kill_process(
                         parent_pid,
                         [(signal.SIGUSR1, 5), (signal.SIGTERM, 15),
@@ -1196,7 +1244,7 @@ def attach_debugger(pid):
     try:
         terminal.communicate()
     except SigUsr1:
-        print("Debugging complete.")
+        print("\nDebugging complete.")
         pass
 
 
