@@ -29,6 +29,22 @@ if PY3:
 else:
     Bytes = ast.Str
 
+
+if sys.version_info >= (3, 8):
+    from ast import TypeIgnore, AsyncFunctionDef
+else:
+
+    # TypeIgnore, AsyncFunctionDef does not exist on Python 3.7 and before. thus
+    # we define a dummy TypeIgnore, AsyncFunctionDef just to simplify remaining
+    # code.
+
+    class TypeIgnore:
+        pass
+
+    class AsyncFunctionDef:
+        pass
+
+
 def _is_comment_or_blank(line):
     """
     Returns whether a line of python code contains only a comment is blank.
@@ -56,6 +72,9 @@ def _flatten_ast_nodes(arg):
         pass
     elif isinstance(arg, ast.AST):
         yield arg
+    elif isinstance(arg, str):
+        #FunctionDef type_comments
+        yield arg
     elif isinstance(arg, (tuple, list, types.GeneratorType)):
         for x in arg:
             for y in _flatten_ast_nodes(x):
@@ -80,23 +99,41 @@ def _iter_child_nodes_in_order(node):
 
 
 def _iter_child_nodes_in_order_internal_1(node):
+    if isinstance(node, str):
+        # this happen for type comments which are not ast nodes but str
+        # they do not have children. We yield nothing.
+        yield []
+        return
     if not isinstance(node, ast.AST):
         raise TypeError
     if isinstance(node, ast.Dict):
         assert node._fields == ("keys", "values")
         yield list(zip(node.keys, node.values))
-    elif isinstance(node, ast.FunctionDef):
+    elif isinstance(node, (ast.FunctionDef, AsyncFunctionDef)):
         if six.PY2:
             assert node._fields == ('name', 'args', 'body', 'decorator_list'), node._fields
             yield node.decorator_list, node.args, node.body
         elif sys.version_info >= (3, 8):
-            assert node._fields == ('name', 'args', 'body', 'decorator_list',
-                                    'returns', 'type_comment'), node._fields
-            yield node.type_comment, node.returns, node.decorator_list, node.args, node.body
+            assert node._fields == (
+                "name",
+                "args",
+                "body",
+                "decorator_list",
+                "returns",
+                "type_comment",
+            ), node._fields
+            res = (
+                node.type_comment,
+                node.decorator_list,
+                node.args,
+                node.returns,
+                node.body,
+            )
+            yield res
         else:
             assert node._fields == ('name', 'args', 'body', 'decorator_list',
                                     'returns'), node._fields
-            yield node.returns, node.decorator_list, node.args, node.body
+            yield node.decorator_list, node.args, node.returns, node.body
         # node.name is a string, not an AST node
     elif isinstance(node, ast.arguments):
         if six.PY2:
@@ -115,6 +152,12 @@ def _iter_child_nodes_in_order_internal_1(node):
     elif isinstance(node, ast.IfExp):
         assert node._fields == ('test', 'body', 'orelse')
         yield node.body, node.test, node.orelse
+    elif isinstance(node, ast.Call):
+        # call arguments order are lost by ast, re-order them
+        yield node.func
+        args = sorted([(k.value.lineno, k.value.col_offset, k) for k in node.keywords]+
+                      [(k.lineno,k.col_offset, k) for k in node.args])
+        yield [a[2] for a in args]
     elif isinstance(node, ast.ClassDef):
         if six.PY2:
             assert node._fields == ('name', 'bases', 'body', 'decorator_list')
@@ -122,7 +165,7 @@ def _iter_child_nodes_in_order_internal_1(node):
             assert node._fields == ('name', 'bases', 'keywords', 'body', 'decorator_list')
         yield node.decorator_list, node.bases, node.body
         # node.name is a string, not an AST node
-    elif sys.version_info >= (3, 6) and isinstance(node, ast.FormattedValue):
+    elif sys.version_info >= (3, 7) and isinstance(node, ast.FormattedValue):
         assert node._fields == ('value', 'conversion', 'format_spec')
         yield node.value,
     else:
@@ -199,7 +242,6 @@ def _parse_ast_nodes(text, flags, auto_flags, mode):
       ``ast.Module``
     """
     text = FileText(text)
-    flags = CompilerFlags(flags)
     filename = str(text.filename) if text.filename else "<unknown>"
     source = text.joined
     source = dedent(source)
@@ -233,7 +275,7 @@ def _test_parse_string_literal(text, flags):
     Attempt to parse ``text``.  If it parses cleanly to a single string
     literal, return its value.  Otherwise return ``None``.
 
-      >>> _test_parse_string_literal(r'"foo\n" r"\nbar"', 0)
+      >>> _test_parse_string_literal(r'"foo\n" r"\nbar"', None)
       'foo\n\\nbar'
 
     """
@@ -327,6 +369,20 @@ def _annotate_ast_startpos(ast_node, parent_ast_node, minpos, text, flags):
     :raise ValueError:
       Could not find the starting line number.
     """
+    assert isinstance(ast_node, (ast.AST, str, TypeIgnore)), ast_node
+
+    # joined strings and children do not carry a column offset on pre-3.8
+    # this prevent reformatting.
+    # set the column offset to the parent value before 3.8
+    if (3, 7) < sys.version_info < (3, 8):
+        if (
+            isinstance(ast_node, (getattr(ast, "JoinedStr", None), ast.FormattedValue))
+            or isinstance(
+                parent_ast_node, (getattr(ast, "JoinedStr", None), ast.FormattedValue)
+            )
+        ) and ast_node.col_offset == -1:
+            ast_node.col_offset = parent_ast_node.col_offset
+
     # First, traverse child nodes.  If the first child node (recursively) is a
     # multiline string, then we need to transfer its information to this node.
     # Walk all nodes/fields of the AST.  We implement this as a custom
@@ -340,7 +396,7 @@ def _annotate_ast_startpos(ast_node, parent_ast_node, minpos, text, flags):
                                          child_minpos, text, flags)
         if is_first_child and leftstr:
             leftstr_node = child_node
-        if hasattr(child_node, 'lineno'):
+        if hasattr(child_node, 'lineno') and not isinstance(child_node, TypeIgnore):
             if child_node.startpos < child_minpos:
                 raise AssertionError(
                     "Got out-of-order AST node(s):\n"
@@ -365,14 +421,14 @@ def _annotate_ast_startpos(ast_node, parent_ast_node, minpos, text, flags):
 
     # If the node has no lineno at all, then skip it.  This should only happen
     # for nodes we don't care about, e.g. ``ast.Module`` or ``ast.alias``.
-    if not hasattr(ast_node, 'lineno'):
+    if not hasattr(ast_node, 'lineno') or isinstance(ast_node, TypeIgnore):
         return False
     # If col_offset is set then the lineno should be correct also.
     if ast_node.col_offset >= 0:
         # In Python 3.8+, FunctionDef.lineno is the line with the def. To
         # account for decorators, we need the lineno of the first decorator
         if (sys.version_info >= (3, 8)
-            and isinstance(ast_node, (ast.FunctionDef, ast.ClassDef))
+            and isinstance(ast_node, (ast.FunctionDef, ast.ClassDef, AsyncFunctionDef))
             and ast_node.decorator_list):
             delta = (ast_node.decorator_list[0].lineno-1,
                      # The col_offset doesn't include the @
@@ -416,6 +472,8 @@ def _annotate_ast_startpos(ast_node, parent_ast_node, minpos, text, flags):
             startpos = FilePos(startpos.lineno, startpos.colno - lk)
             assert str(text[startpos:(startpos+(0,4))]) == "with"
         ast_node.startpos = startpos
+        if sys.version_info <= (3, 8):
+            ast_node.startpos = max(startpos, minpos)
         return False
 
     assert ast_node.col_offset == -1
@@ -445,6 +503,11 @@ def _annotate_ast_startpos(ast_node, parent_ast_node, minpos, text, flags):
 
     # It should now be the case that we are looking at a multi-line string
     # literal.
+    if sys.version_info >= (3, 7) and isinstance(ast_node, ast.FormattedValue):
+        ast_node.startpos = ast_node.value.startpos
+        ast_node.endpos = ast_node.value.startpos
+
+        return True
     if not isinstance(ast_node, (ast.Str, Bytes)):
         raise ValueError(
             "got a non-string col_offset=-1: %s" % (ast.dump(ast_node)))
@@ -468,6 +531,9 @@ def _annotate_ast_startpos(ast_node, parent_ast_node, minpos, text, flags):
             for _m in re.finditer("[bBrRuU]*[\"\']", start_line)])
     target_str = ast_node.s
 
+    if isinstance(target_str, bytes) and sys.version_info[:2] == (3, 7):
+        target_str = target_str.decode()
+
     # Loop over possible end_linenos.  The first one we've identified is the
     # by far most likely one, but in theory it could be anywhere later in the
     # file.  This could be because of a dastardly concatenated string like
@@ -484,6 +550,10 @@ def _annotate_ast_startpos(ast_node, parent_ast_node, minpos, text, flags):
     # have much to go on to figure out that the real end_lineno is 7.  If we
     # don't find the string ending on L3, then search forward looking for the
     # real end of the string.  Yuck!
+    #
+    # This is now complicated by fstrings that do interpolate variable on 3.7 fixed on 3.8+)
+    # where we'll try to guess based on prefix
+    f_string_candidate_prefixes = []
     for end_lineno in range(first_end_lineno, text.endpos.lineno+1):
         # Compute possible end positions.  We're given the line we're ending
         # on, but not the column position.  Note that the ending line could
@@ -528,15 +598,32 @@ def _annotate_ast_startpos(ast_node, parent_ast_node, minpos, text, flags):
             candidate_str = _test_parse_string_literal(subtext, flags)
             if candidate_str is None:
                 continue
-            elif target_str == candidate_str:
+            if isinstance(candidate_str, bytes) and sys.version_info[:2] == (3, 7):
+                candidate_str = candidate_str.decode()
+
+            maybe_fstring = False
+            try:
+                if (3, 7) <= sys.version_info <= (3, 8):
+                    potential_start = text.lines[startpos.lineno - 1]
+                    maybe_fstring = ("f'" in potential_start) or (
+                        'f"' in potential_start
+                    )
+            except IndexError:
+                pass
+
+            if target_str == candidate_str and target_str:
                 # Success!
                 ast_node.startpos = startpos
                 ast_node.endpos   = endpos
                 # This node is a multiline string; and, it's a leaf, so by
                 # definition it is the leftmost node.
-                return True # all done
-            elif target_str.startswith(candidate_str):
+                return True  # all done
+            elif candidate_str and target_str.startswith(candidate_str):
                 matched_prefix.add(startpos)
+            elif maybe_fstring:
+                candidate_prefix = candidate_str.split("{")[0]
+                if candidate_prefix and target_str.startswith(candidate_prefix):
+                    f_string_candidate_prefixes.append((startpos, endpos))
         # We didn't find a string given the current end_lineno candidate.
         # Only continue checking the startpos candidates that so far produced
         # prefixes of the string we're looking for.
@@ -547,9 +634,18 @@ def _annotate_ast_startpos(ast_node, parent_ast_node, minpos, text, flags):
             for (sq, sp) in startpos_candidates
             if sp in matched_prefix
         ]
-    raise ValueError(
-        "Couldn't find exact position of %s"
-        % (ast.dump(ast_node)))
+    if (3, 7) <= sys.version_info <= (3, 8):
+        if len(f_string_candidate_prefixes) == 1:
+            # we did not find the string but there is one fstring candidate starting it
+
+            ast_node.startpos, ast_node.endpos = f_string_candidate_prefixes[0]
+            return True
+        elif isinstance(parent_ast_node, ast.JoinedStr):
+            self_pos = parent_ast_node.values.index(ast_node)
+            ast_node.startpos = parent_ast_node.values[self_pos - 1].startpos
+            ast_node.endpos = parent_ast_node.values[self_pos - 1].endpos
+            return True
+    raise ValueError("Couldn't find exact position of %s" % (ast.dump(ast_node)))
 
 
 def _annotate_ast_context(ast_node):
@@ -561,6 +657,7 @@ def _annotate_ast_context(ast_node):
 
     For non-list fields, the index part is ``None``.
     """
+    assert isinstance(ast_node, ast.AST)
     for field_name, field_value in ast.iter_fields(ast_node):
         if isinstance(field_value, ast.AST):
             child_node = field_value
@@ -663,7 +760,8 @@ def _ast_node_is_in_docstring_position(ast_node):
     it eligible as a docstring.
 
     The main way a ``Str`` can be a docstring is if it is a standalone string
-    at the beginning of a ``Module``, ``FunctionDef``, or ``ClassDef``.
+    at the beginning of a ``Module``, ``FunctionDef``, ``AsyncFucntionDef``
+    or ``ClassDef``.
 
     We also support variable docstrings per Epydoc:
 
@@ -691,7 +789,7 @@ def _ast_node_is_in_docstring_position(ast_node):
     if expr_ctx.field != 'body':
         return False
     parent_node = expr_ctx.parent
-    if not isinstance(parent_node, (ast.FunctionDef, ast.ClassDef, ast.Module)):
+    if not isinstance(parent_node, (ast.FunctionDef, ast.ClassDef, ast.Module, AsyncFunctionDef)):
         return False
     if expr_ctx.index == 0:
         return True
@@ -733,7 +831,7 @@ class PythonStatement(object):
     Representation of a top-level Python statement or consecutive
     comments/blank lines.
 
-      >>> PythonStatement('print("x",\n file=None)\n', flags=0x10000)
+      >>> PythonStatement('print("x",\n file=None)\n', flags='print_function')  #doctest: +SKIP
       PythonStatement('print("x",\n file=None)\n', flags=0x10000)
 
     Implemented as a wrapper around a `PythonBlock` containing at most one
@@ -1277,7 +1375,7 @@ class PythonBlock(object):
         #    if _ast_node_is_in_docstring_position(n)]
         # However, the method we now use is more straightforward, and doesn't
         # require first annotating each node with context information.
-        docstring_containers = (ast.FunctionDef, ast.ClassDef, ast.Module)
+        docstring_containers = (ast.FunctionDef, ast.ClassDef, ast.Module, AsyncFunctionDef)
         for node in _walk_ast_nodes_in_order(self.annotated_ast_node):
             if not isinstance(node, docstring_containers):
                 continue
